class VideoController < ApplicationController
  layout "admin"

  # Make sure that a user logs in before doing any action here
  before_filter :authorize, :except => :login

  def index
    render :action => 'index'
  end

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def list
    @video_pages, @videos = paginate :videos, :per_page => 10
  end

  def show
    @video = Video.find(params[:id])
  end

  def new
    @video = Video.new
  end

  def create
    @video = Video.new(params[:video])
    if @video.save
      flash[:notice] = 'Video was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end

  def edit
    @video = Video.find(params[:id])
  end

  def update
    @video = Video.find(params[:id])
    if @video.update_attributes(params[:video])
      flash[:notice] = 'Video was successfully updated.'
      redirect_to :action => 'show', :id => @video
    else
      render :action => 'edit'
    end
  end

  def destroy
    Video.find(params[:id]).destroy
    redirect_to :action => 'list'
  end

  def search
    if request.post?
      @query = params[:q]
      @videos = Video.find(:all, :conditions => ["title like ?", "%#{@query[0]}%"])
      render :action => 'searchresults'
    else
      render :action => 'search'
    end
  end
end
